package cn.minbb.edu.storage;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.core.io.Resource;
import org.springframework.core.io.UrlResource;
import org.springframework.stereotype.Service;
import org.springframework.util.FileSystemUtils;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.net.MalformedURLException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.stream.Stream;

@Service
public class FileServiceImpl implements FileService {

    private Logger logger = LoggerFactory.getLogger(FileServiceImpl.class);

    @Override
    public void storeToFolder(MultipartFile file, String filename, StorageProperties.Folder folder) {
        try {
            if (file.isEmpty()) {
                throw new StorageException("Failed to store empty file " + file.getOriginalFilename());
            }
            Files.copy(file.getInputStream(), Paths.get(folder.getPath()).resolve(filename));
        } catch (IOException e) {
            throw new StorageException("Failed to store file " + file.getOriginalFilename(), e);
        }
    }

    @Override
    public void deleteFromFolder(String filename, StorageProperties.Folder folder) {
        try {
            Files.delete(loadFromFolder(filename, folder));
        } catch (IOException e) {
            throw new StorageException("Failed to delete file " + filename, e);
        }
    }

    @Override
    public void deleteAll() {
        FileSystemUtils.deleteRecursively(Paths.get(StorageProperties.Folder.LOCATION.getPath()).toFile());
    }

    @Override
    public Path loadFromFolder(String filename, StorageProperties.Folder folder) {
        return Paths.get(folder.getPath()).resolve(filename);
    }

    @Override
    public Stream<Path> loadAllFromFolder(StorageProperties.Folder folder) {
        Path rootLocation = Paths.get(folder.getPath());
        try {
            return Files.walk(rootLocation, 1)
                    .filter(path -> !path.equals(rootLocation))
                    .map(path -> rootLocation.relativize(path));
        } catch (IOException e) {
            throw new StorageException("Failed to read stored files", e);
        }
    }

    @Override
    public Resource loadAsResourceFromFolder(String filename, StorageProperties.Folder folder) {
        try {
            Path file = loadFromFolder(filename, folder);
            Resource resource = new UrlResource(file.toUri());
            if (resource.exists() || resource.isReadable()) {
                return resource;
            } else {
                throw new StorageFileNotFoundException("Could not read file: " + filename);
            }
        } catch (MalformedURLException e) {
            throw new StorageFileNotFoundException("Could not read file: " + filename, e);
        }
    }
}